@@ -5,475 +5,578 @@
 #
 # Copyright 2013 M. J. Dominus.
 # You may copy and distribute this program under the
-# same terms as Perl itself.  
+# same terms as Perl itself.
 # If in doubt, write to mjd-perl-template+@plover.com for a license.
 #
 
 package Text::Template;
-$Text::Template::VERSION = '1.47';
+$Text::Template::VERSION = '1.48';
 # ABSTRACT: Expand template text with embedded Perl
 
-require 5.004;
-use Exporter;
-@ISA = qw(Exporter);
-@EXPORT_OK = qw(fill_in_file fill_in_string TTerror);
-use vars '$ERROR';
 use strict;
+use warnings;
+
+require 5.008;
+
+use base 'Exporter';
+
+our @EXPORT_OK = qw(fill_in_file fill_in_string TTerror);
+our $ERROR;
 
 my %GLOBAL_PREPEND = ('Text::Template' => '');
 
 sub Version {
-  $Text::Template::VERSION;
+    $Text::Template::VERSION;
 }
 
 sub _param {
-  my $kk;
-  my ($k, %h) = @_;
-  for $kk ($k, "\u$k", "\U$k", "-$k", "-\u$k", "-\U$k") {
-    return $h{$kk} if exists $h{$kk};
-  }
-  return;
+    my ($k, %h) = @_;
+
+    for my $kk ($k, "\u$k", "\U$k", "-$k", "-\u$k", "-\U$k") {
+        return $h{$kk} if exists $h{$kk};
+    }
+
+    return undef;
 }
 
-sub always_prepend
-{
-  my $pack = shift;
-  my $old = $GLOBAL_PREPEND{$pack};
-  $GLOBAL_PREPEND{$pack} = shift;
-  $old;
-}
-
-{
-  my %LEGAL_TYPE;
-  BEGIN { 
-    %LEGAL_TYPE = map {$_=>1} qw(FILE FILEHANDLE STRING ARRAY);
-  }
-  sub new {
+sub always_prepend {
     my $pack = shift;
-    my %a = @_;
-    my $stype = uc(_param('type', %a) || "FILE");
-    my $source = _param('source', %a);
-    my $untaint = _param('untaint', %a);
-    my $prepend = _param('prepend', %a);
-    my $alt_delim = _param('delimiters', %a);
-    my $broken = _param('broken', %a);
-    unless (defined $source) {
-      require Carp;
-      Carp::croak("Usage: $ {pack}::new(TYPE => ..., SOURCE => ...)");
-    }
-    unless ($LEGAL_TYPE{$stype}) {
-      require Carp;
-      Carp::croak("Illegal value `$stype' for TYPE parameter");
-    }
-    my $self = {TYPE => $stype,
-		PREPEND => $prepend,
-                UNTAINT => $untaint,
-                BROKEN => $broken,
-		(defined $alt_delim ? (DELIM => $alt_delim) : ()),
-	       };
-    # Under 5.005_03, if any of $stype, $prepend, $untaint, or $broken
-    # are tainted, all the others become tainted too as a result of
-    # sharing the expression with them.  We install $source separately
-    # to prevent it from acquiring a spurious taint.
-    $self->{SOURCE} = $source;
 
-    bless $self => $pack;
-    return unless $self->_acquire_data;
-    
-    $self;
-  }
+    my $old = $GLOBAL_PREPEND{$pack};
+
+    $GLOBAL_PREPEND{$pack} = shift;
+
+    $old;
+}
+
+{
+    my %LEGAL_TYPE;
+
+    BEGIN {
+        %LEGAL_TYPE = map { $_ => 1 } qw(FILE FILEHANDLE STRING ARRAY);
+    }
+
+    sub new {
+        my ($pack, %a) = @_;
+
+        my $stype     = uc(_param('type', %a) || "FILE");
+        my $source    = _param('source', %a);
+        my $untaint   = _param('untaint', %a);
+        my $prepend   = _param('prepend', %a);
+        my $alt_delim = _param('delimiters', %a);
+        my $broken    = _param('broken', %a);
+
+        unless (defined $source) {
+            require Carp;
+            Carp::croak("Usage: $ {pack}::new(TYPE => ..., SOURCE => ...)");
+        }
+
+        unless ($LEGAL_TYPE{$stype}) {
+            require Carp;
+            Carp::croak("Illegal value `$stype' for TYPE parameter");
+        }
+
+        my $self = {
+            TYPE    => $stype,
+            PREPEND => $prepend,
+            UNTAINT => $untaint,
+            BROKEN  => $broken,
+            (defined $alt_delim ? (DELIM => $alt_delim) : ())
+        };
+
+        # Under 5.005_03, if any of $stype, $prepend, $untaint, or $broken
+        # are tainted, all the others become tainted too as a result of
+        # sharing the expression with them.  We install $source separately
+        # to prevent it from acquiring a spurious taint.
+        $self->{SOURCE} = $source;
+
+        bless $self => $pack;
+        return unless $self->_acquire_data;
+
+        $self;
+    }
 }
 
 # Convert template objects of various types to type STRING,
 # in which the template data is embedded in the object itself.
 sub _acquire_data {
-  my ($self) = @_;
-  my $type = $self->{TYPE};
-  if ($type eq 'STRING') {
-    # nothing necessary    
-  } elsif ($type eq 'FILE') {
-    my $data = _load_text($self->{SOURCE});
-    unless (defined $data) {
-      # _load_text already set $ERROR
-      return undef;
+    my $self = shift;
+
+    my $type = $self->{TYPE};
+
+    if ($type eq 'STRING') {
+        # nothing necessary
     }
-    if ($self->{UNTAINT} && _is_clean($self->{SOURCE})) {
-      _unconditionally_untaint($data);
+    elsif ($type eq 'FILE') {
+        my $data = _load_text($self->{SOURCE});
+        unless (defined $data) {
+
+            # _load_text already set $ERROR
+            return undef;
+        }
+        if ($self->{UNTAINT} && _is_clean($self->{SOURCE})) {
+            _unconditionally_untaint($data);
+        }
+        $self->{TYPE}     = 'STRING';
+        $self->{FILENAME} = $self->{SOURCE};
+        $self->{SOURCE}   = $data;
     }
-    $self->{TYPE} = 'STRING';
-    $self->{FILENAME} = $self->{SOURCE};
-    $self->{SOURCE} = $data;
-  } elsif ($type eq 'ARRAY') {
-    $self->{TYPE} = 'STRING';
-    $self->{SOURCE} = join '', @{$self->{SOURCE}};
-  } elsif ($type eq 'FILEHANDLE') {
-    $self->{TYPE} = 'STRING';
-    local $/;
-    my $fh = $self->{SOURCE};
-    my $data = <$fh>; # Extra assignment avoids bug in Solaris perl5.00[45].
-    if ($self->{UNTAINT}) {
-      _unconditionally_untaint($data);
+    elsif ($type eq 'ARRAY') {
+        $self->{TYPE} = 'STRING';
+        $self->{SOURCE} = join '', @{ $self->{SOURCE} };
     }
-    $self->{SOURCE} = $data;
-  } else {
-    # This should have been caught long ago, so it represents a 
-    # drastic `can't-happen' sort of failure
-    my $pack = ref $self;
-    die "Can only acquire data for $pack objects of subtype STRING, but this is $type; aborting";
-  }
-  $self->{DATA_ACQUIRED} = 1;
+    elsif ($type eq 'FILEHANDLE') {
+        $self->{TYPE} = 'STRING';
+        local $/;
+        my $fh   = $self->{SOURCE};
+        my $data = <$fh>;             # Extra assignment avoids bug in Solaris perl5.00[45].
+        if ($self->{UNTAINT}) {
+            _unconditionally_untaint($data);
+        }
+        $self->{SOURCE} = $data;
+    }
+    else {
+        # This should have been caught long ago, so it represents a
+        # drastic `can't-happen' sort of failure
+        my $pack = ref $self;
+        die "Can only acquire data for $pack objects of subtype STRING, but this is $type; aborting";
+    }
+
+    $self->{DATA_ACQUIRED} = 1;
 }
 
 sub source {
-  my ($self) = @_;
-  $self->_acquire_data unless $self->{DATA_ACQUIRED};
-  return $self->{SOURCE};
+    my $self = shift;
+
+    $self->_acquire_data unless $self->{DATA_ACQUIRED};
+
+    return $self->{SOURCE};
 }
 
 sub set_source_data {
-  my ($self, $newdata) = @_;
-  $self->{SOURCE} = $newdata;
-  $self->{DATA_ACQUIRED} = 1;
-  $self->{TYPE} = 'STRING';
-  1;
+    my ($self, $newdata, $type) = @_;
+
+    $self->{SOURCE}        = $newdata;
+    $self->{DATA_ACQUIRED} = 1;
+    $self->{TYPE}          = $type || 'STRING';
+
+    1;
 }
 
 sub compile {
-  my $self = shift;
+    my $self = shift;
 
-  return 1 if $self->{TYPE} eq 'PREPARSED';
+    return 1 if $self->{TYPE} eq 'PREPARSED';
 
-  return undef unless $self->_acquire_data;
-  unless ($self->{TYPE} eq 'STRING') {
-    my $pack = ref $self;
-    # This should have been caught long ago, so it represents a 
-    # drastic `can't-happen' sort of failure
-    die "Can only compile $pack objects of subtype STRING, but this is $self->{TYPE}; aborting";
-  }
+    return undef unless $self->_acquire_data;
 
-  my @tokens;
-  my $delim_pats = shift() || $self->{DELIM};
+    unless ($self->{TYPE} eq 'STRING') {
+        my $pack = ref $self;
 
-  
-
-  my ($t_open, $t_close) = ('{', '}');
-  my $DELIM;			# Regex matches a delimiter if $delim_pats
-  if (defined $delim_pats) {
-    ($t_open, $t_close) = @$delim_pats;
-    $DELIM = "(?:(?:\Q$t_open\E)|(?:\Q$t_close\E))";
-    @tokens = split /($DELIM|\n)/, $self->{SOURCE};
-  } else {
-    @tokens = split /(\\\\(?=\\*[{}])|\\[{}]|[{}\n])/, $self->{SOURCE};
-  }
-  my $state = 'TEXT';
-  my $depth = 0;
-  my $lineno = 1;
-  my @content;
-  my $cur_item = '';
-  my $prog_start;
-  while (@tokens) {
-    my $t = shift @tokens;
-    next if $t eq '';
-    if ($t eq $t_open) {	# Brace or other opening delimiter
-      if ($depth == 0) {
-	push @content, [$state, $cur_item, $lineno] if $cur_item ne '';
-	$cur_item = '';
-	$state = 'PROG';
-	$prog_start = $lineno;
-      } else {
-	$cur_item .= $t;
-      }
-      $depth++;
-    } elsif ($t eq $t_close) {	# Brace or other closing delimiter
-      $depth--;
-      if ($depth < 0) {
-	$ERROR = "Unmatched close brace at line $lineno";
-	return undef;
-      } elsif ($depth == 0) {
-	push @content, [$state, $cur_item, $prog_start] if $cur_item ne '';
-	$state = 'TEXT';
-	$cur_item = '';
-      } else {
-	$cur_item .= $t;
-      }
-    } elsif (!$delim_pats && $t eq '\\\\') { # precedes \\\..\\\{ or \\\..\\\}
-      $cur_item .= '\\';
-    } elsif (!$delim_pats && $t =~ /^\\([{}])$/) { # Escaped (literal) brace?
-	$cur_item .= $1;
-    } elsif ($t eq "\n") {	# Newline
-      $lineno++;
-      $cur_item .= $t;
-    } else {			# Anything else
-      $cur_item .= $t;
+        # This should have been caught long ago, so it represents a
+        # drastic `can't-happen' sort of failure
+        die "Can only compile $pack objects of subtype STRING, but this is $self->{TYPE}; aborting";
     }
-  }
 
-  if ($state eq 'PROG') {
-    $ERROR = "End of data inside program text that began at line $prog_start";
-    return undef;
-  } elsif ($state eq 'TEXT') {
-    push @content, [$state, $cur_item, $lineno] if $cur_item ne '';
-  } else {
-    die "Can't happen error #1";
-  }
-  
-  $self->{TYPE} = 'PREPARSED';
-  $self->{SOURCE} = \@content;
-  1;
+    my @tokens;
+    my $delim_pats = shift() || $self->{DELIM};
+
+    my ($t_open, $t_close) = ('{', '}');
+    my $DELIM;    # Regex matches a delimiter if $delim_pats
+
+    if (defined $delim_pats) {
+        ($t_open, $t_close) = @$delim_pats;
+        $DELIM = "(?:(?:\Q$t_open\E)|(?:\Q$t_close\E))";
+        @tokens = split /($DELIM|\n)/, $self->{SOURCE};
+    }
+    else {
+        @tokens = split /(\\\\(?=\\*[{}])|\\[{}]|[{}\n])/, $self->{SOURCE};
+    }
+
+    my $state  = 'TEXT';
+    my $depth  = 0;
+    my $lineno = 1;
+    my @content;
+    my $cur_item = '';
+    my $prog_start;
+
+    while (@tokens) {
+        my $t = shift @tokens;
+
+        next if $t eq '';
+
+        if ($t eq $t_open && (!defined $delim_pats || $state eq 'TEXT')) {    # Brace or other opening delimiter
+            if ($depth == 0) {
+                push @content, [ $state, $cur_item, $lineno ] if $cur_item ne '';
+                $cur_item   = '';
+                $state      = 'PROG';
+                $prog_start = $lineno;
+            }
+            else {
+                $cur_item .= $t;
+            }
+            $depth++;
+        }
+        elsif ($t eq $t_close) {    # Brace or other closing delimiter
+            $depth--;
+            if ($depth < 0) {
+                $ERROR = "Unmatched close brace at line $lineno";
+                return undef;
+            }
+            elsif ($depth == 0) {
+                push @content, [ $state, $cur_item, $prog_start ] if $cur_item ne '';
+                $state    = 'TEXT';
+                $cur_item = '';
+            }
+            else {
+                $cur_item .= $t;
+            }
+        }
+        elsif (!$delim_pats && $t eq '\\\\') {    # precedes \\\..\\\{ or \\\..\\\}
+            $cur_item .= '\\';
+        }
+        elsif (!$delim_pats && $t =~ /^\\([{}])$/) {    # Escaped (literal) brace?
+            $cur_item .= $1;
+        }
+        elsif ($t eq "\n") {                            # Newline
+            $lineno++;
+            $cur_item .= $t;
+        }
+        else {                                          # Anything else
+            $cur_item .= $t;
+        }
+    }
+
+    if ($state eq 'PROG') {
+        $ERROR = "End of data inside program text that began at line $prog_start";
+        return undef;
+    }
+    elsif ($state eq 'TEXT') {
+        push @content, [ $state, $cur_item, $lineno ] if $cur_item ne '';
+    }
+    else {
+        die "Can't happen error #1";
+    }
+
+    $self->{TYPE}   = 'PREPARSED';
+    $self->{SOURCE} = \@content;
+
+    1;
 }
 
 sub prepend_text {
-  my ($self) = @_;
-  my $t = $self->{PREPEND};
-  unless (defined $t) {
-    $t = $GLOBAL_PREPEND{ref $self};
+    my $self = shift;
+
+    my $t = $self->{PREPEND};
+
     unless (defined $t) {
-      $t = $GLOBAL_PREPEND{'Text::Template'};
+        $t = $GLOBAL_PREPEND{ ref $self };
+        unless (defined $t) {
+            $t = $GLOBAL_PREPEND{'Text::Template'};
+        }
     }
-  }
-  $self->{PREPEND} = $_[1] if $#_ >= 1;
-  return $t;
+
+    $self->{PREPEND} = $_[1] if $#_ >= 1;
+
+    return $t;
 }
 
 sub fill_in {
-  my $fi_self = shift;
-  my %fi_a = @_;
+    my ($fi_self, %fi_a) = @_;
 
-  unless ($fi_self->{TYPE} eq 'PREPARSED') {
-    my $delims = _param('delimiters', %fi_a);
-    my @delim_arg = (defined $delims ? ($delims) : ());
-    $fi_self->compile(@delim_arg)
-      or return undef;
-  }
-
-  my $fi_varhash = _param('hash', %fi_a);
-  my $fi_package = _param('package', %fi_a) ;
-  my $fi_broken  = 
-    _param('broken', %fi_a)  || $fi_self->{BROKEN} || \&_default_broken;
-  my $fi_broken_arg = _param('broken_arg', %fi_a) || [];
-  my $fi_safe = _param('safe', %fi_a);
-  my $fi_ofh = _param('output', %fi_a);
-  my $fi_eval_package;
-  my $fi_scrub_package = 0;
-  my $fi_filename = _param('filename') || $fi_self->{FILENAME} || 'template';
-
-  my $fi_prepend = _param('prepend', %fi_a);
-  unless (defined $fi_prepend) {
-    $fi_prepend = $fi_self->prepend_text;
-  }
-
-  if (defined $fi_safe) {
-    $fi_eval_package = 'main';
-  } elsif (defined $fi_package) {
-    $fi_eval_package = $fi_package;
-  } elsif (defined $fi_varhash) {
-    $fi_eval_package = _gensym();
-    $fi_scrub_package = 1;
-  } else {
-    $fi_eval_package = caller;
-  }
-
-  my $fi_install_package;
-  if (defined $fi_varhash) {
-    if (defined $fi_package) {
-      $fi_install_package = $fi_package;
-    } elsif (defined $fi_safe) {
-      $fi_install_package = $fi_safe->root;
-    } else {
-      $fi_install_package = $fi_eval_package; # The gensymmed one
+    unless ($fi_self->{TYPE} eq 'PREPARSED') {
+        my $delims = _param('delimiters', %fi_a);
+        my @delim_arg = (defined $delims ? ($delims) : ());
+        $fi_self->compile(@delim_arg)
+            or return undef;
     }
-    _install_hash($fi_varhash => $fi_install_package);
-  }
 
-  if (defined $fi_package && defined $fi_safe) {
-    no strict 'refs';
-    # Big fat magic here: Fix it so that the user-specified package
-    # is the default one available in the safe compartment.
-    *{$fi_safe->root . '::'} = \%{$fi_package . '::'};   # LOD
-  }
+    my $fi_varhash    = _param('hash',       %fi_a);
+    my $fi_package    = _param('package',    %fi_a);
+    my $fi_broken     = _param('broken',     %fi_a) || $fi_self->{BROKEN} || \&_default_broken;
+    my $fi_broken_arg = _param('broken_arg', %fi_a) || [];
+    my $fi_safe       = _param('safe',       %fi_a);
+    my $fi_ofh        = _param('output',     %fi_a);
+    my $fi_filename   = _param('filename',   %fi_a) || $fi_self->{FILENAME} || 'template';
+    my $fi_strict     = _param('strict',     %fi_a);
+    my $fi_prepend    = _param('prepend',    %fi_a);
 
-  my $fi_r = '';
-  my $fi_item;
-  foreach $fi_item (@{$fi_self->{SOURCE}}) {
-    my ($fi_type, $fi_text, $fi_lineno) = @$fi_item;
-    if ($fi_type eq 'TEXT') {
-      $fi_self->append_text_to_output(
-        text   => $fi_text,
-        handle => $fi_ofh,
-        out    => \$fi_r,
-        type   => $fi_type,
-      );
-    } elsif ($fi_type eq 'PROG') {
-      no strict;
-      my $fi_lcomment = "#line $fi_lineno $fi_filename";
-      my $fi_progtext = 
-        "package $fi_eval_package; $fi_prepend;\n$fi_lcomment\n$fi_text;";
-      my $fi_res;
-      my $fi_eval_err = '';
-      if ($fi_safe) {
-        $fi_safe->reval(q{undef $OUT});
-	$fi_res = $fi_safe->reval($fi_progtext);
-	$fi_eval_err = $@;
-	my $OUT = $fi_safe->reval('$OUT');
-	$fi_res = $OUT if defined $OUT;
-      } else {
-	my $OUT;
-	$fi_res = eval $fi_progtext;
-	$fi_eval_err = $@;
-	$fi_res = $OUT if defined $OUT;
-      }
+    my $fi_eval_package;
+    my $fi_scrub_package = 0;
 
-      # If the value of the filled-in text really was undef,
-      # change it to an explicit empty string to avoid undefined
-      # value warnings later.
-      $fi_res = '' unless defined $fi_res;
-
-      if ($fi_eval_err) {
-	$fi_res = $fi_broken->(text => $fi_text,
-			       error => $fi_eval_err,
-			       lineno => $fi_lineno,
-			       arg => $fi_broken_arg,
-			       );
-	if (defined $fi_res) {
-          $fi_self->append_text_to_output(
-            text   => $fi_res,
-            handle => $fi_ofh,
-            out    => \$fi_r,
-            type   => $fi_type,
-          );
-	} else {
-	  return $fi_res;		# Undefined means abort processing
-	}
-      } else {
-        $fi_self->append_text_to_output(
-          text   => $fi_res,
-          handle => $fi_ofh,
-          out    => \$fi_r,
-          type   => $fi_type,
-        );
-      }
-    } else {
-      die "Can't happen error #2";
+    unless (defined $fi_prepend) {
+        $fi_prepend = $fi_self->prepend_text;
     }
-  }
 
-  _scrubpkg($fi_eval_package) if $fi_scrub_package;
-  defined $fi_ofh ? 1 : $fi_r;
+    if (defined $fi_safe) {
+        $fi_eval_package = 'main';
+    }
+    elsif (defined $fi_package) {
+        $fi_eval_package = $fi_package;
+    }
+    elsif (defined $fi_varhash) {
+        $fi_eval_package  = _gensym();
+        $fi_scrub_package = 1;
+    }
+    else {
+        $fi_eval_package = caller;
+    }
+
+    my @fi_varlist;
+    my $fi_install_package;
+
+    if (defined $fi_varhash) {
+        if (defined $fi_package) {
+            $fi_install_package = $fi_package;
+        }
+        elsif (defined $fi_safe) {
+            $fi_install_package = $fi_safe->root;
+        }
+        else {
+            $fi_install_package = $fi_eval_package;    # The gensymmed one
+        }
+        @fi_varlist = _install_hash($fi_varhash => $fi_install_package);
+        if ($fi_strict) {
+            $fi_prepend = "use vars qw(@fi_varlist);$fi_prepend" if @fi_varlist;
+            $fi_prepend = "use strict;$fi_prepend";
+        }
+    }
+
+    if (defined $fi_package && defined $fi_safe) {
+        no strict 'refs';
+
+        # Big fat magic here: Fix it so that the user-specified package
+        # is the default one available in the safe compartment.
+        *{ $fi_safe->root . '::' } = \%{ $fi_package . '::' };    # LOD
+    }
+
+    my $fi_r = '';
+    my $fi_item;
+    foreach $fi_item (@{ $fi_self->{SOURCE} }) {
+        my ($fi_type, $fi_text, $fi_lineno) = @$fi_item;
+        if ($fi_type eq 'TEXT') {
+            $fi_self->append_text_to_output(
+                text   => $fi_text,
+                handle => $fi_ofh,
+                out    => \$fi_r,
+                type   => $fi_type,);
+        }
+        elsif ($fi_type eq 'PROG') {
+            no strict;
+
+            my $fi_lcomment = "#line $fi_lineno $fi_filename";
+            my $fi_progtext = "package $fi_eval_package; $fi_prepend;\n$fi_lcomment\n$fi_text\n;";
+            my $fi_res;
+            my $fi_eval_err = '';
+
+            if ($fi_safe) {
+                $fi_safe->reval(q{undef $OUT});
+                $fi_res      = $fi_safe->reval($fi_progtext);
+                $fi_eval_err = $@;
+                my $OUT = $fi_safe->reval('$OUT');
+                $fi_res = $OUT if defined $OUT;
+            }
+            else {
+                my $OUT;
+                $fi_res      = eval $fi_progtext;
+                $fi_eval_err = $@;
+                $fi_res      = $OUT if defined $OUT;
+            }
+
+            # If the value of the filled-in text really was undef,
+            # change it to an explicit empty string to avoid undefined
+            # value warnings later.
+            $fi_res = '' unless defined $fi_res;
+
+            if ($fi_eval_err) {
+                $fi_res = $fi_broken->(
+                    text   => $fi_text,
+                    error  => $fi_eval_err,
+                    lineno => $fi_lineno,
+                    arg    => $fi_broken_arg,);
+                if (defined $fi_res) {
+                    $fi_self->append_text_to_output(
+                        text   => $fi_res,
+                        handle => $fi_ofh,
+                        out    => \$fi_r,
+                        type   => $fi_type,);
+                }
+                else {
+                    return $fi_r;    # Undefined means abort processing
+                }
+            }
+            else {
+                $fi_self->append_text_to_output(
+                    text   => $fi_res,
+                    handle => $fi_ofh,
+                    out    => \$fi_r,
+                    type   => $fi_type,);
+            }
+        }
+        else {
+            die "Can't happen error #2";
+        }
+    }
+
+    _scrubpkg($fi_eval_package) if $fi_scrub_package;
+
+    defined $fi_ofh ? 1 : $fi_r;
 }
 
 sub append_text_to_output {
-  my ($self, %arg) = @_;
+    my ($self, %arg) = @_;
 
-  if (defined $arg{handle}) {
-    print { $arg{handle} } $arg{text};
-  } else {
-    ${ $arg{out} } .= $arg{text};
-  }
+    if (defined $arg{handle}) {
+        print { $arg{handle} } $arg{text};
+    }
+    else {
+        ${ $arg{out} } .= $arg{text};
+    }
 
-  return;
+    return;
 }
 
 sub fill_this_in {
-  my $pack = shift;
-  my $text = shift;
-  my $templ = $pack->new(TYPE => 'STRING', SOURCE => $text, @_)
-    or return undef;
-  $templ->compile or return undef;
-  my $result = $templ->fill_in(@_);
-  $result;
+    my ($pack, $text) = splice @_, 0, 2;
+
+    my $templ = $pack->new(TYPE => 'STRING', SOURCE => $text, @_)
+        or return undef;
+
+    $templ->compile or return undef;
+
+    my $result = $templ->fill_in(@_);
+
+    $result;
 }
 
 sub fill_in_string {
-  my $string = shift;
-  my $package = _param('package', @_);
-  push @_, 'package' => scalar(caller) unless defined $package;
-  Text::Template->fill_this_in($string, @_);
+    my $string = shift;
+
+    my $package = _param('package', @_);
+
+    push @_, 'package' => scalar(caller) unless defined $package;
+
+    Text::Template->fill_this_in($string, @_);
 }
 
 sub fill_in_file {
-  my $fn = shift;
-  my $templ = Text::Template->new(TYPE => 'FILE', SOURCE => $fn, @_)
-    or return undef;
-  $templ->compile or return undef;
-  my $text = $templ->fill_in(@_);
-  $text;
+    my $fn = shift;
+    my $templ = Text::Template->new(TYPE => 'FILE', SOURCE => $fn, @_) or return undef;
+
+    $templ->compile or return undef;
+
+    my $text = $templ->fill_in(@_);
+
+    $text;
 }
 
 sub _default_broken {
-  my %a = @_;
-  my $prog_text = $a{text};
-  my $err = $a{error};
-  my $lineno = $a{lineno};
-  chomp $err;
-#  $err =~ s/\s+at .*//s;
-  "Program fragment delivered error ``$err''";
+    my %a = @_;
+
+    my $prog_text = $a{text};
+    my $err       = $a{error};
+    my $lineno    = $a{lineno};
+
+    chomp $err;
+
+    #  $err =~ s/\s+at .*//s;
+    "Program fragment delivered error ``$err''";
 }
 
 sub _load_text {
-  my $fn = shift;
-  local *F;
-  unless (open F, $fn) {
-    $ERROR = "Couldn't open file $fn: $!";
-    return undef;
-  }
-  local $/;
-  <F>;
+    my $fn = shift;
+
+    open my $fh, '<', $fn or do {
+        $ERROR = "Couldn't open file $fn: $!";
+        return undef;
+    };
+
+    local $/;
+
+    <$fh>;
 }
 
 sub _is_clean {
-  my $z;
-  eval { ($z = join('', @_)), eval '#' . substr($z,0,0); 1 }   # LOD
+    my $z;
+
+    eval { ($z = join('', @_)), eval '#' . substr($z, 0, 0); 1 }    # LOD
 }
 
 sub _unconditionally_untaint {
-  for (@_) {
-    ($_) = /(.*)/s;
-  }
+    for (@_) {
+        ($_) = /(.*)/s;
+    }
 }
 
 {
-  my $seqno = 0;
-  sub _gensym {
-    __PACKAGE__ . '::GEN' . $seqno++;
-  }
-  sub _scrubpkg {
-    my $s = shift;
-    $s =~ s/^Text::Template:://;
-    no strict 'refs';
-    my $hash = $Text::Template::{$s."::"};
-    foreach my $key (keys %$hash) {
-      undef $hash->{$key};
+    my $seqno = 0;
+
+    sub _gensym {
+        __PACKAGE__ . '::GEN' . $seqno++;
+    }
+
+    sub _scrubpkg {
+        my $s = shift;
+
+        $s =~ s/^Text::Template:://;
+
+        no strict 'refs';
+
+        my $hash = $Text::Template::{ $s . "::" };
+
+        foreach my $key (keys %$hash) {
+            undef $hash->{$key};
+        }
+
+        %$hash = ();
+
+        delete $Text::Template::{ $s . "::" };
     }
-    %$hash = ();
-    delete $Text::Template::{$s."::"};
-  }
 }
-  
+
 # Given a hashful of variables (or a list of such hashes)
 # install the variables into the specified package,
 # overwriting whatever variables were there before.
 sub _install_hash {
-  my $hashlist = shift;
-  my $dest = shift;
-  if (UNIVERSAL::isa($hashlist, 'HASH')) {
-    $hashlist = [$hashlist];
-  }
-  my $hash;
-  foreach $hash (@$hashlist) {
-    my $name;
-    foreach $name (keys %$hash) {
-      my $val = $hash->{$name};
-      no strict 'refs';
-      local *SYM = *{"$ {dest}::$name"};
-      if (! defined $val) {
-	delete ${"$ {dest}::"}{$name};
-      } elsif (ref $val) {
-	*SYM = $val;
-      } else {
- 	*SYM = \$val;
-      }
+    my $hashlist = shift;
+    my $dest     = shift;
+
+    if (UNIVERSAL::isa($hashlist, 'HASH')) {
+        $hashlist = [$hashlist];
     }
-  }
+
+    my @varlist;
+
+    for my $hash (@$hashlist) {
+        for my $name (keys %$hash) {
+            my $val = $hash->{$name};
+
+            no strict 'refs';
+
+            local *SYM = *{"$ {dest}::$name"};
+
+            if (!defined $val) {
+                delete ${"$ {dest}::"}{$name};
+                my $match = qr/^.\Q$name\E$/;
+                @varlist = grep { $_ !~ $match } @varlist;
+            }
+            elsif (ref $val) {
+                *SYM = $val;
+                push @varlist, do {
+                    if    (UNIVERSAL::isa($val, 'ARRAY')) { '@' }
+                    elsif (UNIVERSAL::isa($val, 'HASH'))  { '%' }
+                    else                                  { '$' }
+                    }
+                    . $name;
+            }
+            else {
+                *SYM = \$val;
+                push @varlist, '$' . $name;
+            }
+        }
+    }
+
+    @varlist;
 }
 
 sub TTerror { $ERROR }
@@ -490,7 +593,7 @@ Text::Template - Expand template text with embedded Perl
 
 =head1 VERSION
 
-version 1.47
+version 1.48
 
 =head1 SYNOPSIS
 
@@ -904,7 +1007,7 @@ may yield spurious warnings about
 so you might like to avoid them and use the capitalized versions.
 
 At present, there are eight legal options:  C<PACKAGE>, C<BROKEN>,
-C<BROKEN_ARG>, C<SAFE>, C<HASH>, C<OUTPUT>, and C<DELIMITERS>.
+C<BROKEN_ARG>, C<FILENAME>, C<SAFE>, C<HASH>, C<OUTPUT>, and C<DELIMITERS>.
 
 =over 4
 
@@ -1183,6 +1286,29 @@ message into C<$error> this way.  Then the function that called
 C<fill_in> can see if C<my_broken> has left an error message for it
 to find, and proceed accordingly.
 
+=item C<FILENAME>
+
+If you give C<fill_in> a C<FILENAME> option, then this is the file name that
+you loaded the template source from.  This only affects the error message that
+is given for template errors.  If you loaded the template from C<foo.txt> for
+example, and pass C<foo.txt> as the C<FILENAME> parameter, errors will look
+like C<... at foo.txt line N> rather than C<... at template line N>. 
+
+Note that this does NOT have anything to do with loading a template from the
+given filename.  See C<fill_in_file()> for that.
+
+For example:
+
+ my $template = Text::Template->new(
+     TYPE   => 'string',
+     SOURCE => 'The value is {1/0}');
+
+ $template->fill_in(FILENAME => 'foo.txt') or die $Text::Template::ERROR;
+
+will die with an error that contains
+
+ Illegal division by zero at at foo.txt line 1
+
 =item C<SAFE>
 
 If you give C<fill_in> a C<SAFE> option, its value should be a safe
@@ -1526,7 +1652,7 @@ been declared.  C<use strict> was implied, even though you did not
 write it explicitly, because the C<PREPEND> option added it for you
 automatically.
 
-There are two other ways to do this.  At the time you create the
+There are three other ways to do this.  At the time you create the
 template object with C<new>, you can also supply a C<PREPEND> option,
 in which case the statements will be prepended each time you fill in
 that template.  If the C<fill_in> call has its own C<PREPEND> option,
@@ -1539,6 +1665,33 @@ If you do this, then call calls to C<fill_in> for I<any> template will
 attach the perl statements to the beginning of each program fragment,
 except where overridden by C<PREPEND> options to C<new> or C<fill_in>.
 
+An alternative to adding "use strict;" to the PREPEND option, you can
+pass STRICT => 1 to fill_in when also passing the HASH option.
+
+Suppose that the C<fill_in> call included both
+
+	HASH => {$foo => ''} and
+	STRICT => 1
+
+options, and that the template looked like this:
+
+	{ 
+	  $foo = 14;
+	  ...
+	}
+
+	...
+
+	{ my $result = $boo + 12;    # $boo is misspelled and should be $foo
+	  ...
+	}
+
+The code in the second fragment would fail, because C<$boo> has not
+been declared. C<use strict> was implied, even though you did not
+write it explicitly, because the C<STRICT> option added it for you
+automatically. Any variable referenced in the template that is not in the
+C<HASH> option will be an error.
+
 =head2 Prepending in Derived Classes
 
 This section is technical, and you should skip it on the first few
@@ -1818,11 +1971,12 @@ method.  It is passed a list of pairs with these entries:
   text   - the text that will be appended
   type   - where the text came from: TEXT for literal text, PROG for code
 
-=head1 SUPPORT
+=head1 HISTORY
 
-This software may have bugs.  Suggestions and bug reports are always welcome.
-Send them to C<mjd-perl-template+@plover.com>.  (That is my address, not the
-address of the mailing list.  The mailing list address is a secret.)
+Originally written by Mark Jason Dominus, Plover Systems (versions 0.01 - 1.46)
+
+Maintainership transferred to Michael Schout E<lt>mschout@cpan.orgE<gt> in version
+1.47
 
 =head1 THANKS
 
@@ -1935,26 +2089,21 @@ There are not quite enough tests in the test suite.
 
 =head1 SOURCE
 
-The development version is on github at L<http://github.com/mschout/perl-text-template>
-and may be cloned from L<git://github.com/mschout/perl-text-template.git>
+The development version is on github at L<http://https://github.com/mschout/perl-text-template>
+and may be cloned from L<git://https://github.com/mschout/perl-text-template.git>
 
 =head1 BUGS
 
-Please report any bugs or feature requests to bug-text-template@rt.cpan.org or through the web interface at:
- http://rt.cpan.org/Public/Dist/Display.html?Name=Text-Template
+Please report any bugs or feature requests on the bugtracker website
+L<https://github.com/mschout/perl-text-template/issues>
+
+When submitting a bug or request, please include a test-file or a
+patch to an existing test-file that illustrates the bug or desired
+feature.
 
 =head1 AUTHOR
 
-Mark Jason Dominus, Plover Systems
-
-Please send questions and other remarks about this software to
-C<mjd-perl-template+@plover.com>
-
-You can join a very low-volume (E<lt>10 messages per year) mailing
-list for announcements about this package.  Send an empty note to
-C<mjd-perl-template-request@plover.com> to join.
-
-For updates, visit C<http://www.plover.com/~mjd/perl/Template/>.
+Michael Schout <mschout@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
